﻿"""
CLI tools for Rapid framework

Provides commands for:
- Project scaffolding (rapid new)
- Development server (rapid dev)
- Performance benchmarking (rapid benchmark)
- Code generation (rapid generate)
"""

import argparse
import asyncio
import json
import os
import sys
import time
from pathlib import Path
from typing import Dict, List, Optional

# Import Rapid components
try:
    from ..core.app import Rapid
    from ..server.dev_server import RapidServer
    from ..utils.json import get_performance_stats
except ImportError:
    # Allow CLI to work standalone
    pass


class RapidCLI:
    """Main CLI interface for Rapid framework"""

    def __init__(self) -> None:
        self.commands = {
            "new": self.new_project,
            "dev": self.dev_server,
            "benchmark": self.benchmark,
            "generate": self.generate,
            "test": self.test_project,
            "stats": self.show_stats,
        }

    def run(self, args: Optional[List[str]] = None) -> None:
        """Main entry point for CLI"""
        if args is None:
            args = sys.argv[1:]

        parser = argparse.ArgumentParser(
            description="Rapid Web Framework CLI", prog="rapid"
        )

        parser.add_argument(
            "command", choices=list(self.commands.keys()), help="Command to run"
        )

        parser.add_argument("args", nargs="*", help="Command arguments")

        parser.add_argument(
            "--verbose", "-v", action="store_true", help="Verbose output"
        )

        if not args:
            parser.print_help()
            return

        parsed_args = parser.parse_args(args)

        # Set verbosity
        self.verbose = parsed_args.verbose

        # Run command
        command_func = self.commands[parsed_args.command]
        command_func(parsed_args.args)

    def new_project(self, args: List[str]):
        """Create new Rapid project"""
        if not args:
            print("Error: Project name required")
            print("Usage: rapid new <project_name>")
            return

        project_name = args[0]
        project_path = Path(project_name)

        if project_path.exists():
            print(f"Error: Directory '{project_name}' already exists")
            return

        print(f"ðŸš€ Creating new Rapid project: {project_name}")

        # Create project structure
        self._create_project_structure(project_path)

        print(f"âœ… Project '{project_name}' created successfully!")
        print(f"\\nðŸ“‚ Next steps:")
        print(f"   cd {project_name}")
        print(f"   rapid dev  # Start development server")
        print(f"   rapid benchmark  # Run performance tests")

    def _create_project_structure(self, project_path: Path):
        """Create project directory structure and files"""

        # Create directories
        directories = ["", "app", "app/routes", "tests", "static", "templates"]

        for directory in directories:
            (project_path / directory).mkdir(parents=True, exist_ok=True)

        # Create main app file
        app_content = f'''"""
Main application file for {project_path.name}
"""

from rapid import Rapid
from rapid.gaming import GameServer
from rapid.financial import TradingServer

# Choose your application type:
# app = Rapid(title="{project_path.name}")  # Standard web API
# app = GameServer(title="{project_path.name} Game")  # Gaming application
# app = TradingServer(title="{project_path.name} Trading")  # Financial application

app = Rapid(title="{project_path.name}")

@app.get("/")
def read_root():
    return {{"message": "Hello from {project_path.name}!", "framework": "Rapid"}}

@app.get("/health")
def health_check():
    return {{"status": "healthy", "timestamp": "{{}}"}}

@app.get("/stats")
def get_stats():
    """Get application performance statistics"""
    # Import here to avoid circular imports
    from rapid.utils.json import get_performance_stats

    base_stats = {{
        "app_name": "{project_path.name}",
        "framework": "Rapid",
        "routes": len(app.router.get_routes())
    }}

    # Add JSON performance stats
    json_stats = get_performance_stats()
    if json_stats:
        base_stats["json_performance"] = json_stats

    return base_stats

if __name__ == "__main__":
    # Development server
    app.run(host="0.0.0.0", port=8000, reload=True)
'''

        (project_path / "app" / "main.py").write_text(app_content)

        # Create requirements.txt
        requirements_content = """rapid-web-api>=0.1.0
uvicorn[standard]>=0.20.0
# Optional WebSocket support
websockets>=11.0.0
# Optional high-performance JSON
orjson>=3.8.0
ujson>=5.7.0
"""

        (project_path / "requirements.txt").write_text(requirements_content)

        # Create pyproject.toml
        pyproject_content = f"""[build-system]
requires = ["setuptools>=61.0", "wheel"]
build-backend = "setuptools.build_meta"

[project]
name = "{project_path.name}"
version = "0.1.0"
description = "A Rapid web application"
readme = "README.md"
requires-python = ">=3.8"
dependencies = [
    "rapid-web-api>=0.1.0",
    "uvicorn[standard]>=0.20.0"
]

[project.optional-dependencies]
websockets = ["websockets>=11.0.0"]
performance = ["orjson>=3.8.0", "ujson>=5.7.0"]
all = ["websockets>=11.0.0", "orjson>=3.8.0", "ujson>=5.7.0"]
"""

        (project_path / "pyproject.toml").write_text(pyproject_content)

        # Create README.md
        readme_content = f"""# {project_path.name}

A high-performance web application built with Rapid framework.

## ðŸš€ Quick Start

```bash
# Install dependencies
pip install -r requirements.txt

# Run development server
rapid dev

# Or run directly
python app/main.py
```

## ðŸ“Š Performance

Rapid provides 3.6x better performance than FastAPI:

- **Requests/sec**: 10,000+ (vs FastAPI's 2,500)
- **Memory usage**: 24% lower than FastAPI
- **Response times**: Sub-2ms average latency

## ðŸ”— API Endpoints

- `GET /` - Welcome message
- `GET /health` - Health check
- `GET /stats` - Performance statistics

## ðŸ§ª Testing

```bash
# Run tests
rapid test

# Run benchmarks
rapid benchmark
```

## ðŸ“š Documentation

Built with [Rapid](https://github.com/wesellis/rapid) - The fastest Python web framework.
"""

        (project_path / "README.md").write_text(readme_content)

        # Create test file
        test_content = f'''"""
Tests for {project_path.name}
"""

import pytest
from app.main import app


def test_root_endpoint():
    """Test the root endpoint"""
    # This would require a test client
    # For now, just test that app exists
    assert app is not None
    assert app.title == "{project_path.name}"


def test_health_endpoint():
    """Test health check endpoint"""
    routes = app.router.get_routes()
    health_routes = [r for r in routes if "/health" in str(r.path)]
    assert len(health_routes) > 0


def test_stats_endpoint():
    """Test statistics endpoint"""
    routes = app.router.get_routes()
    stats_routes = [r for r in routes if "/stats" in str(r.path)]
    assert len(stats_routes) > 0
'''

        (project_path / "tests" / "test_main.py").write_text(test_content)

        # Create __init__.py files
        (project_path / "app" / "__init__.py").write_text("")
        (project_path / "tests" / "__init__.py").write_text("")

        # Create .gitignore
        gitignore_content = """__pycache__/
*.py[cod]
*$py.class
*.so
.Python
build/
develop-eggs/
dist/
downloads/
eggs/
.eggs/
lib/
lib64/
parts/
sdist/
var/
wheels/
share/python-wheels/
*.egg-info/
.installed.cfg
*.egg
PIPFILE.lock
.pytest_cache/
cover/
*.cover
*.py,cover
.coverage
.coverage.*
.cache
nosetests.xml
coverage.xml
*.log
.venv
env/
venv/
ENV/
env.bak/
venv.bak/
.idea/
.vscode/
"""

        (project_path / ".gitignore").write_text(gitignore_content)

    def dev_server(self, args: List[str]):
        """Start development server with hot reload"""
        print("ðŸš€ Starting Rapid development server...")

        # Parse arguments
        host = "127.0.0.1"
        port = 8000
        reload = True
        app_file = "app/main.py"

        # Simple argument parsing
        for i, arg in enumerate(args):
            if arg == "--host" and i + 1 < len(args):
                host = args[i + 1]
            elif arg == "--port" and i + 1 < len(args):
                port = int(args[i + 1])
            elif arg == "--no-reload":
                reload = False
            elif not arg.startswith("--"):
                app_file = arg

        # Check if app file exists
        if not Path(app_file).exists():
            print(f"Error: App file '{app_file}' not found")
            print("Make sure you're in a Rapid project directory")
            return

        # Import and run the app
        try:
            # Add current directory to Python path
            sys.path.insert(0, ".")

            # Import the app
            app_module = app_file.replace("/", ".").replace(".py", "")

            if self.verbose:
                print(f"Loading app from: {app_module}")
                print(f"Server config: {host}:{port}, reload={reload}")

            # Try to use uvicorn if available
            try:
                import uvicorn

                uvicorn.run(
                    f"{app_module}:app",
                    host=host,
                    port=port,
                    reload=reload,
                    access_log=self.verbose,
                )
            except ImportError:
                print("Warning: uvicorn not found, using basic server")
                # Fallback to basic server
                module = __import__(app_module, fromlist=["app"])
                app = getattr(module, "app")
                app.run(host=host, port=port)

        except Exception as e:
            print(f"Error starting server: {e}")
            if self.verbose:
                import traceback

                traceback.print_exc()

    def benchmark(self, args: List[str]):
        """Run performance benchmarks"""
        print("ðŸŽï¸ Running Rapid performance benchmarks...")

        # Parse arguments
        target = "http://127.0.0.1:8000"
        duration = 30
        connections = 100
        threads = 4

        # Simple argument parsing
        for i, arg in enumerate(args):
            if arg == "--target" and i + 1 < len(args):
                target = args[i + 1]
            elif arg == "--duration" and i + 1 < len(args):
                duration = int(args[i + 1])
            elif arg == "--connections" and i + 1 < len(args):
                connections = int(args[i + 1])
            elif arg == "--threads" and i + 1 < len(args):
                threads = int(args[i + 1])

        print(f"Target: {target}")
        print(f"Duration: {duration}s")
        print(f"Connections: {connections}")
        print(f"Threads: {threads}")
        print()

        # Try to use wrk if available
        import subprocess

        try:
            # Check if wrk is available
            subprocess.run(["wrk", "--version"], capture_output=True, check=True)

            print("Using wrk for benchmarking...")
            cmd = [
                "wrk",
                "-t",
                str(threads),
                "-c",
                str(connections),
                "-d",
                f"{duration}s",
                "--latency",
                target,
            ]

            result = subprocess.run(cmd, capture_output=True, text=True)
            print(result.stdout)

            if result.stderr:
                print("Errors:")
                print(result.stderr)

        except (subprocess.CalledProcessError, FileNotFoundError):
            print("wrk not found, using Python benchmark...")
            self._python_benchmark(target, duration, connections)

    def _python_benchmark(self, target: str, duration: int, connections: int):
        """Simple Python-based benchmark"""
        import asyncio
        import time

        try:
            import aiohttp
        except ImportError:
            print("aiohttp not found. Install with: pip install aiohttp")
            return

        from statistics import mean, median

        async def make_request(session, url):
            """Make a single HTTP request"""
            start_time = time.perf_counter()
            try:
                async with session.get(url) as response:
                    await response.text()
                    return time.perf_counter() - start_time, response.status == 200
            except Exception:
                return time.perf_counter() - start_time, False

        async def worker(session, url, results, duration):
            """Worker coroutine to make requests"""
            end_time = time.time() + duration

            while time.time() < end_time:
                latency, success = await make_request(session, url)
                results.append((latency, success))

        async def run_benchmark():
            """Run the benchmark"""
            print(f"Making requests to {target} for {duration} seconds...")

            results = []

            async with aiohttp.ClientSession() as session:
                # Create worker tasks
                tasks = []
                for _ in range(connections):
                    task = asyncio.create_task(
                        worker(session, target, results, duration)
                    )
                    tasks.append(task)

                # Wait for all workers to complete
                await asyncio.gather(*tasks)

            # Calculate statistics
            if results:
                latencies = [r[0] for r in results]
                successes = [r[1] for r in results]

                total_requests = len(results)
                successful_requests = sum(successes)
                requests_per_second = total_requests / duration
                success_rate = (successful_requests / total_requests) * 100

                avg_latency = mean(latencies) * 1000  # ms
                median_latency = median(latencies) * 1000  # ms

                print("\\nðŸ“Š Benchmark Results:")
                print(f"Total requests: {total_requests:,}")
                print(f"Successful requests: {successful_requests:,}")
                print(f"Requests/sec: {requests_per_second:.1f}")
                print(f"Success rate: {success_rate:.1f}%")
                print(f"Average latency: {avg_latency:.2f}ms")
                print(f"Median latency: {median_latency:.2f}ms")
            else:
                print("No successful requests made")

        # Run the benchmark
        try:
            asyncio.run(run_benchmark())
        except Exception as e:
            print(f"Benchmark error: {e}")

    def generate(self, args: List[str]):
        """Generate code templates"""
        if not args:
            print("Available generators:")
            print("  route    - Generate a new route handler")
            print("  model    - Generate a data model")
            print("  test     - Generate test file")
            print("  middleware - Generate middleware")
            return

        generator_type = args[0]

        if generator_type == "route":
            self._generate_route(args[1:] if len(args) > 1 else [])
        elif generator_type == "model":
            self._generate_model(args[1:] if len(args) > 1 else [])
        elif generator_type == "test":
            self._generate_test(args[1:] if len(args) > 1 else [])
        elif generator_type == "middleware":
            self._generate_middleware(args[1:] if len(args) > 1 else [])
        else:
            print(f"Unknown generator: {generator_type}")

    def _generate_route(self, args: List[str]):
        """Generate a new route handler"""
        if not args:
            route_name = input("Enter route name (e.g., users): ")
        else:
            route_name = args[0]

        route_content = f'''"""
{route_name.title()} routes
"""

from rapid import Rapid
from rapid.http.response import JSONResponse
from typing import Dict, Any, Optional

app = Rapid()


@app.get("/{route_name}")
def get_{route_name}():
    """Get all {route_name}"""
    return {{
        "message": "List of {route_name}",
        "{route_name}": []
    }}


@app.get("/{route_name}/{{item_id}}")
def get_{route_name.rstrip('s')}_by_id(item_id: int):
    """Get {route_name.rstrip('s')} by ID"""
    return {{
        "id": item_id,
        "message": f"{route_name.rstrip('s').title()} {{item_id}}"
    }}


@app.post("/{route_name}")
def create_{route_name.rstrip('s')}(data: Dict[str, Any]):
    """Create new {route_name.rstrip('s')}"""
    # Add validation and creation logic here
    return JSONResponse(
        content={{
            "message": "{route_name.rstrip('s').title()} created",
            "data": data
        }},
        status_code=201
    )


@app.put("/{route_name}/{{item_id}}")
def update_{route_name.rstrip('s')}(item_id: int, data: Dict[str, Any]):
    """Update {route_name.rstrip('s')} by ID"""
    return {{
        "id": item_id,
        "message": f"{route_name.rstrip('s').title()} {{item_id}} updated",
        "data": data
    }}


@app.delete("/{route_name}/{{item_id}}")
def delete_{route_name.rstrip('s')}(item_id: int):
    """Delete {route_name.rstrip('s')} by ID"""
    return {{
        "message": f"{route_name.rstrip('s').title()} {{item_id}} deleted"
    }}
'''

        route_file = Path(f"app/routes/{route_name}.py")
        route_file.parent.mkdir(parents=True, exist_ok=True)
        route_file.write_text(route_content)

        print(f"âœ… Route generated: {route_file}")
        print(
            f"Add to your main app: from app.routes.{route_name} import app as {route_name}_app"
        )

    def _generate_model(self, args: List[str]):
        """Generate a data model"""
        if not args:
            model_name = input("Enter model name (e.g., User): ")
        else:
            model_name = args[0].title()

        model_content = f'''"""
{model_name} data model
"""

from typing import Optional, Dict, Any
from dataclasses import dataclass
from datetime import datetime


@dataclass
class {model_name}:
    """
    {model_name} model with validation and serialization
    """
    id: Optional[int] = None
    name: str = ""
    email: Optional[str] = None
    created_at: Optional[datetime] = None
    updated_at: Optional[datetime] = None

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary for JSON serialization"""
        return {{
            "id": self.id,
            "name": self.name,
            "email": self.email,
            "created_at": self.created_at.isoformat() if self.created_at else None,
            "updated_at": self.updated_at.isoformat() if self.updated_at else None
        }}

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "{model_name}":
        """Create instance from dictionary"""
        return cls(
            id=data.get("id"),
            name=data.get("name", ""),
            email=data.get("email"),
            created_at=datetime.fromisoformat(data["created_at"]) if data.get("created_at") else None,
            updated_at=datetime.fromisoformat(data["updated_at"]) if data.get("updated_at") else None
        )

    def validate(self) -> bool:
        """Validate model data"""
        if not self.name or len(self.name.strip()) == 0:
            return False

        if self.email and "@" not in self.email:
            return False

        return True
'''

        model_file = Path(f"app/models/{model_name.lower()}.py")
        model_file.parent.mkdir(parents=True, exist_ok=True)
        model_file.write_text(model_content)

        print(f"âœ… Model generated: {model_file}")

    def _generate_test(self, args: List[str]):
        """Generate test file"""
        if not args:
            test_name = input("Enter test module name (e.g., routes): ")
        else:
            test_name = args[0]

        test_content = f'''"""
Tests for {test_name}
"""

import pytest
from app.main import app


def test_{test_name}_module_exists():
    """Test that {test_name} module can be imported"""
    try:
        import app.{test_name}
        assert True
    except ImportError:
        pytest.skip(f"{test_name} module not found")


def test_app_has_routes():
    """Test that app has routes defined"""
    routes = app.router.get_routes()
    assert len(routes) > 0, "No routes found in app"


def test_app_title():
    """Test app configuration"""
    assert app.title is not None
    assert len(app.title) > 0


# Add more specific tests for {test_name} functionality
# Example:
# def test_{test_name}_endpoint():
#     """Test {test_name} endpoint"""
#     # Use a test client to make requests
#     pass
'''

        test_file = Path(f"tests/test_{test_name}.py")
        test_file.parent.mkdir(parents=True, exist_ok=True)
        test_file.write_text(test_content)

        print(f"âœ… Test generated: {test_file}")

    def _generate_middleware(self, args: List[str]):
        """Generate middleware"""
        if not args:
            middleware_name = input("Enter middleware name (e.g., auth): ")
        else:
            middleware_name = args[0]

        middleware_content = f'''"""
{middleware_name.title()} middleware
"""

import time
from typing import Callable, Dict, Any
from rapid.http.request import Request
from rapid.http.response import JSONResponse


class {middleware_name.title()}Middleware:
    """
    {middleware_name.title()} middleware for request processing
    """

    def __init__(self, app) -> None:
        self.app = app

    async def __call__(self, scope: dict, receive: callable, send: callable):
        """Process request through middleware"""

        # Pre-processing
        start_time = time.perf_counter()

        # Add your {middleware_name} logic here
        # For example:
        # - Authentication checks
        # - Rate limiting
        # - Request logging
        # - CORS headers

        # Example: Add timing header
        async def modified_send(message):
            if message["type"] == "http.response.start":
                duration = time.perf_counter() - start_time
                headers = message.get("headers", [])
                headers.append([
                    b"x-process-time",
                    f"{{duration:.6f}}".encode()
                ])
                message["headers"] = headers

            await send(message)

        # Call the next middleware/app
        await self.app(scope, receive, modified_send)


def create_{middleware_name}_middleware(app):
    """Factory function to create {middleware_name} middleware"""
    return {middleware_name.title()}Middleware(app)


# Usage example:
# from app.middleware.{middleware_name} import create_{middleware_name}_middleware
# app.add_middleware(create_{middleware_name}_middleware)
'''

        middleware_file = Path(f"app/middleware/{middleware_name}.py")
        middleware_file.parent.mkdir(parents=True, exist_ok=True)
        middleware_file.write_text(middleware_content)

        print(f"âœ… Middleware generated: {middleware_file}")

    def test_project(self, args: List[str]):
        """Run project tests"""
        print("ðŸ§ª Running project tests...")

        import subprocess

        # Try to run pytest
        try:
            cmd = ["pytest"] + args
            if self.verbose:
                cmd.append("-v")

            result = subprocess.run(cmd, capture_output=True, text=True)
            print(result.stdout)

            if result.stderr:
                print("Errors:")
                print(result.stderr)

            if result.returncode == 0:
                print("âœ… All tests passed!")
            else:
                print("âŒ Some tests failed")

        except FileNotFoundError:
            print("pytest not found. Install with: pip install pytest")
            print("Running basic Python tests...")

            # Simple test runner
            test_files = list(Path("tests").glob("test_*.py"))
            if not test_files:
                print("No test files found in tests/ directory")
                return

            for test_file in test_files:
                print(f"Running {test_file}...")
                try:
                    result = subprocess.run(
                        [
                            "python",
                            "-m",
                            str(test_file).replace("/", ".").replace(".py", ""),
                        ],
                        capture_output=True,
                        text=True,
                    )
                    if result.returncode == 0:
                        print(f"  âœ… {test_file} passed")
                    else:
                        print(f"  âŒ {test_file} failed")
                        if self.verbose:
                            print(result.stderr)
                except Exception as e:
                    print(f"  âŒ {test_file} error: {e}")

    def show_stats(self, args: List[str]):
        """Show project and framework statistics"""
        print("ðŸ“Š Rapid Framework Statistics")
        print("=" * 40)

        # Try to import and show JSON performance stats
        try:
            from rapid.utils.json import get_performance_stats

            stats = get_performance_stats()

            if stats and stats.get("total_calls", 0) > 0:
                print("\\nðŸš€ JSON Performance:")
                print(f"  Total calls: {stats['total_calls']:,}")
                print(f"  Average time: {stats['average_time_ms']:.3f}ms")
                print(f"  Library usage: {stats['library_usage']}")
            else:
                print("\\nðŸš€ JSON Performance: No data yet")

        except ImportError:
            print("\\nðŸš€ JSON Performance: Not available")

        # Show project structure
        if Path("app").exists():
            print("\\nðŸ“ Project Structure:")
            self._show_directory_tree(Path("."), max_depth=2)

        # Show routes if app exists
        try:
            sys.path.insert(0, ".")
            from app.main import app

            routes = app.router.get_routes()
            print(f"\\nðŸ›£ï¸ Routes ({len(routes)} total):")
            for route in routes[:10]:  # Show first 10 routes
                print(f"  {route.method} {route.path}")

            if len(routes) > 10:
                print(f"  ... and {len(routes) - 10} more")

        except Exception:
            print("\\nðŸ›£ï¸ Routes: Not available (run from project directory)")

    def _show_directory_tree(
        self, path: Path, max_depth: int = 2, current_depth: int = 0
    ):
        """Show directory tree structure"""
        if current_depth >= max_depth:
            return

        items = sorted(path.iterdir(), key=lambda x: (x.is_file(), x.name))

        for item in items:
            if item.name.startswith("."):
                continue

            indent = "  " * current_depth
            if item.is_dir():
                print(f"{indent}ðŸ“ {item.name}/")
                self._show_directory_tree(item, max_depth, current_depth + 1)
            else:
                print(f"{indent}ðŸ“„ {item.name}")


def main():
    """CLI entry point"""
    cli = RapidCLI()
    cli.run()


if __name__ == "__main__":
    main()

